import { AxiosRequestConfig, Method } from 'axios';
import { API } from '../utils';

/* REQUEST_EXPORTS */

export const request = (
  method: Method,
  url: string,
  data: any = null,
  headers: any = null
) => {
  return API().request({ method, url, data, headers } as AxiosRequestConfig);
};

export interface Request {
  status: 'idle' | 'loading' | 'success' | 'failed';
}

export interface RequestErrors {
  status: number;
  title: string;
  detail: string;
}

export interface FailedRequest {
  errors: RequestErrors;
}

export interface GetData<M> {
  type?: string | null;
  id?: number | null;
  attributes: M;
}

export interface GetAllRequest<M> {
  data: GetData<M>[];
}

export interface GetOneRequest<M> {
  data: GetData<M>;
}

export interface UpsertRelationshipsData<RL = null> {
  attributes?: RL | null;
}

export interface UpsertRelationshipsRequest<RL = null> {
  data?: UpsertRelationshipsData<RL> | null;
}

export interface UpsertData<RQ, RL = null> {
  attributes: RQ;
  relationships?: UpsertRelationshipsRequest<RL> | null;
}

export interface UpsertRequest<RQ, RL = null> {
  data: UpsertData<RQ, RL>;
}

export interface DeleteAttributes {
  success: boolean;
}

export interface DeleteData {
  attributes: DeleteAttributes;
}

export interface DeleteRequest {
  data: DeleteData;
}
